'*****************************************************************************************************
' runSequence.vbs -- a script to make a sequence of exposures and record their average ADU values
'
' Copyright Hilary Jones, 2010
' hdjones@pacbell.net
'
' This script runs a sequence of exposures.  The script is intended to gather the data needed
' to evaluate a CCD's linearity.  It does this by taking a sequence of exposures like this:
'	E0, E1, E0, E2, E0, E3, E0, E4, ..., En, E0
' where the En's are exposure times.  We surround each exposure time by a short reference exposure E0,
' which will be used to compensate for drift in the light box intensity and/or camera.  The value of 
' E0 is a constant, while the values E1, E2, ... are set by using AddTime method.  E0 should be short 
' enough to keep the run short, but not so short that shutter jitter can lead to non-repeatable 
' measurements.
'
' The script can optionally also reverse the sequence of exposures to give an additional way of
' compensating for lightbox drift.  For example, if the AddTime command has been used to enter
' values E1 thru En, the sequence would be:
'	E0, E1, E0, E2, ..., En, E0, E0, En, E0, En-1, E0, ..., E1, E0
' Note that it is not necessary to enter values for the last half of the sequence.
'
' By default the script does not save the FITS images, but rather analyzes them on the fly.  It 
' writes the results to a file containing values in the form:
'	A0, A1, A0
'	A0, A2, A0
'	A0, A3, A0
'	...
' where A0 signfies the value of a reference exposures taken before or after a measurement, 
' and A1, A2, A3... signify the values for the measurements themselves.  Other data is also
' written to the file.  The format is designed to be pasted directly into Excel, where it
' can be analyzed.
'
' The script assumes that MaxIm DL's "Set Calibration" command has been used to define a dark and
' bias frame.  They must be a full frame images, even though the script is only measuring subframes.  
' MaxIm DL will know how to apply the files to the subframe.  Note: both dark and bias frames
' should be prepared so that dark frame scaling can work right.  For the dark frame, choose an 
' exposure time that's greater than the largest value in the exposures array; and be sure to set 
' MDL so it uses auto-scaling or auto-optimize (recommended).
'
' Future:
'	No plans currently
'
'*****************************************************************************************************

Option Explicit

'*****************************************************************************************************
' Variables that may need to be changed for each run:
'	path			where images and/or data files will be saved
'	fileName		name of the data file produced by this script
'	E			list of exposure times, set by calling AddTime
'	E0			reference exposure time
'	size			size of the subframe (if any)
'	filterNumber		the filter number to use (needs to be changed for different cameras!)
'	repeatCount		how many times to run each sequence (ascending + descending counts as one)
'	doingReverse		True => do reverse sequence too
'	savingFiles		True => save all exposures (not recommended except for debugging)
'	usingSubFrames		True => use subframes
'*****************************************************************************************************

' Set up file locations
Const path = "C:\Post Processing\Raw Data\"
Const fileName = "sequenceData.txt"


' Set up list of exposure times
Dim E()			' The list
Dim lastElement		' The index for the last exposure time in the list
lastElement = -1	' Array is initially empty

Call AddTime(1)
Call AddTime(200)
Call AddTime(250)
Call AddTime(300)
Call AddTime(400)
Call AddTime(500)
Call AddTime(600)


' Define the reference exposure time
Const E0 = 5

' Define the size of the subframe
Const size = 400

' Define the filter to be used.  This is a slot number, and may change for different cameras!
Const filterNumber = 3	' 3 => Luminosity for the ST-10

' Define the repeat count.  Ascending + descending counts as one.
Const repeatCount = 1

' Define whether to do the reverse sequence
Const doingReverse = True

' Define whether to save all image files (not recommended)
Const savingFiles = False

' Define whether to use subframes
Const usingSubFrames = True

'*****************************************************************************************************
' Main part of program
'*****************************************************************************************************

Dim I, N		' General purpose loop variables

' Set up output file
Dim fso
Set fso = CreateObject("Scripting.FileSystemObject")
Dim output
Set output = fso.CreateTextFile(path & fileName, True)	' True => can overwrite
output.WriteLine("Pass, E, Direction, Reference before, ADU+Pedestal, Reference after")

' Set up the camera
Dim cam ' "The" Camera object
Set cam = CreateObject("MaxIm.CCDCamera")
cam.LinkEnabled = True
cam.DisableAutoShutdown = True			' Leave camera on when we are done

if Not cam.LinkEnabled Then
   wscript.echo "Failed to start camera."
   Quit
End If

' If using subframes, set up location and size
If (usingSubFrames) Then
Dim left, top
	left = (cam.CameraXSize - size)/2
	top = (cam.CameraYSize - size)/2
	cam.NumX = size
	cam.NumY = size
	cam.StartX = left
	cam.StartY = top
End If

Dim fileNumber		' file number
fileNumber = 1

' Estimate how long this will take.  Add a little time for the overhead between exposures.
Const overhead = 1.2	' Overhead, in seconds
Dim sum
sum = E0 + overhead
For I = 0 to lastElement
	sum = sum + E(I) + overhead + E0 + overhead
Next
If (doingReverse) Then
	sum = sum * 2
End If
sum = sum * repeatCount

Dim ok
ok = MsgBox("This run will take about " & Round(sum/60+0.5) & " minutes.  Press OK to start.", VBOKCancel)
If (ok = VBCancel) Then
	wscript.Quit
End If

' Take the exposures, starting off with a reference image, then alternating measurements w/ reference images
Dim previousRef, result, nextRef
For N = 1 to repeatCount
	previousRef = ADU(E0)
	For I = 0 to lastElement
		result = ADU(E(I))
		nextRef = ADU(E0)
		output.WriteLine(N & ", " & E(I) & ", Up, " & previousRef & ", " & result & ", " & nextRef)
		previousRef = nextRef
	Next
	If (doingReverse) Then
		For I = lastElement to 0 Step -1
			result = ADU(E(I))
			nextRef = ADU(E0)
			output.WriteLine(N & ", " & E(I) & ", Down, " & previousRef & ", " & result & ", " & nextRef)
			previousRef = nextRef
		Next
	End If
Next

output.Close
wscript.echo "Test done"


'************************************************************************
' A subroutine to make it easy to build a list of exposure times
Sub AddTime(time)
	lastElement = lastElement + 1
	Redim Preserve E(lastElement)
	E(lastElement) = time
End Sub


'************************************************************************
' A function for taking an image with a given exposure time, and finding the average ADU value
Function ADU(time)
	cam.Expose time, 1, filterNumber' exposure time, shutter open, filter
	Do While Not cam.ImageReady
		' Don't consume CPU time while waiting
		wscript.sleep 100
	Loop

	' Save image files
	If (savingFiles) Then
		cam.SaveImage(path & "test_" & fileNumber & "_" & time & ".fit")
		fileNumber = fileNumber + 1
	End If

	' Calibrate image using the "Set calibration" command's entries
	cam.Calibrate
	
	' Find average value
	Dim doc
	Set doc = cam.Document
	' CalcAreaInfo returns variant array whose second element is the average
	ADU = doc.CalcAreaInfo(0, 0, doc.XSize-1, doc.YSize-1)(2)
End Function
